<?php

/**
 * Provide a admin area view for the plugin
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @since      1.1.0
 *
 * @package    Shortcodes_finder
 * @subpackage Shortcodes_finder/admin/partials
 */

/**
 * Find posts ids by post type.
 *
 * @since    1.2.9
 */
function sf_get_posts_ids( $post_type = 'post' ) {
	
	$args = array(
		'posts_per_page' => -1,
		'post_type' => $post_type,
		'post_status' => 'publish',
		'orderby' => 'date',
		'order' => 'DESC',
		'fields' => 'ids'
	);
	$posts = get_posts( $args );
	
	return $posts;
	
}

/**
 * Get shortcodes in every content of "type_name" type. List results by shortcode.
 * Allow to pass an existing $shortcodes array to fill with results.
 * Passing a specific shortcode_to_search, returns only that type of occurrences.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_in_content( $type_name, $type_title = '', &$shortcodes = array(), $shortcode_to_search = '' ){
	
	if ($type_title == '') $type_title = ucfirst($type_name);
	
	$args = array(
		'posts_per_page' => -1,
		'post_status' => 'publish',
		'orderby' => 'date',
		'post_type' => $type_name,
		'order' => 'DESC'
	);
	$posts = get_posts( $args );
	sf_get_shortcodes_by_shortcode($posts, $type_title, $shortcodes, $shortcode_to_search);
	
}

/**
 * Find shortcodes in a post recursively, using a specific pattern. Returns it divided into posts.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_by_post_recursive( $content, $post, $pattern, &$shortcodes ) {
	
	if ( preg_match_all( $pattern, $content, $matches ) && array_key_exists( 2, $matches ) ){
		
		foreach( $matches[0] as $key => $value) {
			$shortcode = array();
			
			// replace space with '&' for parse_str() function
			parse_str(str_replace(" ", "&" , $matches[3][$key] ), $params);
			
			$shortcode = array( 'name' => $matches[2][$key], 'params' => $params, 'params_raw' => $matches[3][$key], 'content' => $matches[5][$key], 'code' => $value,
				'post' => array( 'ID' => $post->ID, 'title' => $post->post_title, 'guid' => $post->guid, 'type' => $post->post_type ) );
			
			$shortcodes[$post->ID][] = $shortcode;
			
			// Recursivity
			sf_get_shortcodes_by_post_recursive($matches[5][$key], $post, $pattern, $shortcodes);
		}
	}
}

/**
 * Find all shortcodes in an array of posts recursively. Returns it divided into posts.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_by_post( $posts ){
	
	$shortcodes = array();
	
	sf_import_third_party_shortcodes();
	
	$pattern = '/'. get_shortcode_regex() .'/s';
    
	foreach ($posts as $post){
		
		sf_get_shortcodes_by_post_recursive($post->post_content, $post, $pattern, $shortcodes);
    }
	return $shortcodes;
}

/**
 * Find shortcodes in a post recursively, using a specific pattern (and allowing to specify a single shortcode to find). Returns it divided into shortcode name.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_by_shortcode_recursive ( $content, $post, $type_title, $pattern, &$shortcodes, $shortcode_to_search = '' ) {

	if ( preg_match_all( $pattern, $content, $matches ) && array_key_exists( 2, $matches ) ) {

		foreach( $matches[0] as $key => $value) {
			
			// If no specific shortcode is passed or the current shortcode matched the passed one, add it to the results.
			if ( strlen($shortcode_to_search) == 0 || ( $shortcode_to_search == $matches[2][$key] ) ){
			
				$shortcode = array();
				
				// replace space with '&' for parse_str() function
				parse_str( str_replace(" ", "&" , $matches[3][$key] ), $params );
				
				$shortcode = array( 'name' => $matches[2][$key], 'params' => $params, 'params_raw' => $matches[3][$key], 'content' => $matches[5][$key], 'code' => $value,
					'post' => array( 'ID' => $post->ID, 'title' => $post->post_title, 'type' => (isset($type_title) ? $type_title : $post->post_type), 'guid' => $post->guid ) );

				$shortcodes[$matches[2][$key]][] = $shortcode;
			}
				
			// Recursivity
			sf_get_shortcodes_by_shortcode_recursive($matches[5][$key], $post, $type_title, $pattern, $shortcodes, $shortcode_to_search);
		}
	}
}

/**
 * Find all shortcodes in an array of posts recursively. Returns it divided into shortcode name. Allow to specify a single shortcode to find.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_by_shortcode( $posts, $type_title, &$shortcodes = array(), $shortcode_to_search = '' ){

	sf_import_third_party_shortcodes();
	
	$pattern = '/'. get_shortcode_regex() .'/s';
	
	foreach ($posts as $post)
		sf_get_shortcodes_by_shortcode_recursive($post->post_content, $post, $type_title, $pattern, $shortcodes, $shortcode_to_search);
}

/**
 * Find all unused shortcodes in an array of posts.
 *
 * @since    1.2.9
 */
function sf_get_shortcodes_unused_by_post( $posts ){
	
	$pattern = '/'. sf_get_shortcode_unused_regex() .'/s';
	
	foreach ($posts as $post){
		
		if ( preg_match_all( $pattern, $post->post_content, $matches ) && array_key_exists( 2, $matches ) )
		{
			foreach( $matches[0] as $key => $value) {
				$shortcode = array();
				
				// replace space with '&' for parse_str() function
				parse_str( str_replace(" ", "&" , $matches[3][$key] ), $params );
				
				$shortcode = array( 'name' => $matches[2][$key], 'params' => $params, 'params_raw' => $matches[3][$key], 'content' => $matches[5][$key], 'code' => $value,
					'post' => array( 'ID' => $post->ID, 'title' => $post->post_title, 'guid' => $post->guid ) );
				
				$shortcodes[$matches[2][$key]][] = $shortcode;
			}
		}
    }
	return $shortcodes;
}

/**
 * Import third party shortcodes into $shortcode_tags global system variable.
 * Supported external shortcodes:
 * - WPBakery Visual Composer
 *
 * @since    1.2.9
 */
function sf_import_third_party_shortcodes(){
	
	// Add WPBakery Visual Composer shortcodes
	if ( class_exists('WPBMap') && method_exists('WPBMap', 'addAllMappedShortcodes') )
		WPBMap::addAllMappedShortcodes();
	
}

/**
 * Retrieve the regex pattern to find unused shortcodes.
 *
 * @since    1.2.9
 */
function sf_get_shortcode_unused_regex() {
	
	/* Original regex from Wordpress Core
	return
		  '\\['                              // Opening bracket
		. '(\\[?)'                           // 1: Optional second opening bracket for escaping shortcodes: [[tag]]
		. "($tagregexp)"                     // 2: Shortcode name
		. '(?![\\w-])'                       // Not followed by word character or hyphen
		. '('                                // 3: Unroll the loop: Inside the opening shortcode tag
		.     '[^\\]\\/]*'                   // Not a closing bracket or forward slash
		.     '(?:'
		.         '\\/(?!\\])'               // A forward slash not followed by a closing bracket
		.         '[^\\]\\/]*'               // Not a closing bracket or forward slash
		.     ')*?'
		. ')'
		. '(?:'
		.     '(\\/)'                        // 4: Self closing tag ...
		.     '\\]'                          // ... and closing bracket
		. '|'
		.     '\\]'                          // Closing bracket
		.     '(?:'
		.         '('                        // 5: Unroll the loop: Optionally, anything between the opening and closing shortcode tags
		.             '[^\\[]*+'             // Not an opening bracket
		.             '(?:'
		.                 '\\[(?!\\/\\2\\])' // An opening bracket not followed by the closing shortcode tag
		.                 '[^\\[]*+'         // Not an opening bracket
		.             ')*+'
		.         ')'
		.         '\\[\\/\\2\\]'             // Closing shortcode tag
		.     ')?'
		. ')'
		. '(\\]?)';                          // 6: Optional second closing brocket for escaping shortcodes: [[tag]]
	*/
	
	sf_import_third_party_shortcodes();
	
	global $shortcode_tags;
	$tagnames = array_keys($shortcode_tags);
	$tagregexp = join( '|', array_map('preg_quote', $tagnames) );
	
	$regex = "\["                              // Opening bracket
		. "(\[?)"                           // 1: Optional second opening bracket for escaping shortcodes: [[tag]]
		
		//. "($tagregexp)"                     // 2: Shortcode name
		
		. "(?!\/)"                       // SCRIBIT: Not starting with slash
		. "(?!$tagregexp$)"                     // 2: Shortcode name different from registered shortcodes
		
		//. "(?![\w-])"                       // Not followed by word character or hyphen
		. "("                                // 3: Unroll the loop: Inside the opening shortcode tag
		.     "[^\]\/]*"                   // Not a closing bracket or forward slash
		.     "(?:"
		.         "\/(?!\])"               // A forward slash not followed by a closing bracket
		.         "[^\]\/]*"               // Not a closing bracket or forward slash
		.     ")*?"
		. ")"
		. "(?:"
		.     "(\/)"                        // 4: Self closing tag ...
		.     "\]"                          // ... and closing bracket
		. "|"
		.     "\]"                          // Closing bracket
		.     "(?:"
		.         "("                        // 5: Unroll the loop: Optionally, anything between the opening and closing shortcode tags
		.             "[^\[]*+"             // Not an opening bracket
		.             "(?:"
		.                 "\[(?!\/\2\])" // An opening bracket not followed by the closing shortcode tag
		.                 "[^\[]*+"         // Not an opening bracket
		.             ")*+"
		.         ")"
		.         "\[\/\2\]"             // Closing shortcode tag
		.     ")?"
		. ")"
		. "(\]?)";
	
	return $regex; 
}

/**
 * Escape html text for webpage printing.
 *
 * @since    1.2.9
 */
function sf_html_to_text($html){
	$html = str_replace('<', '&lt;', $html);
	$html = str_replace('>', '&gt;', $html);
	return $html;
}
?>